@@ -14,9 +14,9 @@ our %EXPORT_TAGS = (
     bool => [qw/ true false to_bool /],
 );
 
-our $VERSION = "v0.0.6";
+our $VERSION = "v0.0.7";
 
-sub to_bool :prototype(_) ($x) { $x ? true : false }
+sub to_bool :prototype(_) { $_[0] ? true : false }
 
 sub eqq ($x, $y) {
     return !defined $y unless defined $x;
