@@ -26,14 +26,14 @@ use Carp 'croak';
 
 use experimental 'postderef';
 
-our $VERSION = "v0.0.6";
+our $VERSION = "v0.0.7";
 
 use constant {
     DEFAULT_HEARTBEAT_INTERVAL       => 5,
     DEFAULT_HEARTBEAT_TIMEOUT        => 10,
     DEFAULT_PREFIX                   => 'bs',
     DEFAULT_PING_INTERVAL            => 15,
-    DEFAULT_NOTIFY_PALOAD_SIZE_LIMIT => 8000,
+    DEFAULT_NOTIFY_PAYLOAD_SIZE_LIMIT => 8000,
 };
 
 my $WORKERS_CHANNEL = '_bs:workers';
@@ -46,7 +46,7 @@ sub register ($self, $app, $config) {
     my $heartbeat_timeout = $config->{heartbeat_timeout} // DEFAULT_HEARTBEAT_TIMEOUT;
     my $prefix = $config->{prefix} // DEFAULT_PREFIX;
     my $ping_interval = $config->{ping_interval} // DEFAULT_PING_INTERVAL;
-    my $notify_payload_size_limit = ($config->{notify_payload_size_limit} // DEFAULT_NOTIFY_PALOAD_SIZE_LIMIT) - 1;
+    my $notify_payload_size_limit = ($config->{notify_payload_size_limit} // DEFAULT_NOTIFY_PAYLOAD_SIZE_LIMIT) - 1;
 
     # Database stuff
     my $db_string = $config->{'db_string'} or die "missing db_string configuration option";
@@ -59,6 +59,11 @@ sub register ($self, $app, $config) {
         SELECT pg_get_serial_sequence('event_patch', 'id')
     ")->array->[0];
 
+    my sub get_time {
+        return $app->$prefix->db->query("SELECT EXTRACT(EPOCH FROM CURRENT_TIMESTAMP)")
+            ->array->[0];
+    }
+
     my $pubsub_connected_o = rx_behavior_subject->new(0);
     my $pubsub_connected;
 
@@ -95,21 +100,21 @@ sub register ($self, $app, $config) {
                 delete $state->{$worker_uuid};
 
                 return undef, $state;
-            });
+            }, no_ban => 1);
         });
 
         # register heartbeat
         $app->$prefix->lock_state($WORKERS_CHANNEL, sub ($state) {
-            $last_heartbeat_dt = time;
+            $last_heartbeat_dt = int get_time;
             $state->{$worker_uuid} = {
                 uuid      => $worker_uuid,
                 pid       => $$,
-                hb_expiry => $last_heartbeat_dt + calculate_hb_expiry_dur,
+                heartbeat => $last_heartbeat_dt,
                 hostname  => hostname,
                 banned    => false,
             };
             return undef, $state;
-        });
+        }, no_ban => 1);
         rx_merge(
             rx_timer(rand($heartbeat_interval), $heartbeat_interval),
         )->pipe(
@@ -121,10 +126,10 @@ sub register ($self, $app, $config) {
                     Mojo::IOLoop->stop_gracefully;
                     return undef, undef;
                 }
-                $last_heartbeat_dt = time;
-                $me->{hb_expiry} = $last_heartbeat_dt + calculate_hb_expiry_dur;
+                $last_heartbeat_dt = int get_time;
+                $me->{heartbeat} = $last_heartbeat_dt;
                 return undef, $state;
-            });
+            }, no_ban => 1);
         });
 
         $pubsub_connected_o->subscribe(sub {
@@ -203,16 +208,6 @@ sub register ($self, $app, $config) {
 
     $app->helper("$prefix.lock_state", sub ($c, $channel_names, $sub, %opts) {
         # opts can be: no_txn, no_ban
-        if (
-            $last_heartbeat_dt and
-            time > $last_heartbeat_dt + calculate_hb_expiry_dur() / 2 and
-            ! $opts{no_ban}
-        ) {
-            my $workers_state = $c->$prefix->get_state($WORKERS_CHANNEL);
-            $workers_state->{$worker_uuid}
-                and ! $workers_state->{$worker_uuid}{banned}
-                or croak "worker is banned from lock_state";
-        }
         my $multi_mode = length ref $channel_names;
         $channel_names = [$channel_names] if not $multi_mode;
         my $db = $c->$prefix->db;
@@ -222,6 +217,19 @@ sub register ($self, $app, $config) {
             { name => {-in => $channel_names} },
             { for => 'update' },
         )->hashes; # NOTE: is it possible that the row does not exist?
+
+        # if (
+        #     $last_heartbeat_dt and
+        #         time > $last_heartbeat_dt + calculate_hb_expiry_dur() / 2 and
+        #         ! $opts{no_ban}
+        # )
+        if (! $opts{no_ban}) {
+            my $workers_state = $c->$prefix->get_state($WORKERS_CHANNEL);
+            $workers_state->{$worker_uuid}
+                and ! $workers_state->{$worker_uuid}{banned}
+                or die "worker is banned from lock_state";
+        }
+
         my %rows = map {( $_->{name}, $_ )} @$rows;
         my @rows = @rows{@$channel_names};
         # TODO: IMPORTANT! throw a machine-readable, JSON-able exception if any of the @rows are undef.
@@ -555,9 +563,9 @@ sub register ($self, $app, $config) {
 
     $app->helper("$prefix.global_cleanup", sub ($c) {
         my $workers_state = $c->$prefix->get_state($WORKERS_CHANNEL);
-        my $time = Time::HiRes::time;
+        my $time = get_time;
         foreach my $worker (values %$workers_state) {
-            if ($worker->{hb_expiry} < $time) {
+            if ($time > $worker->{heartbeat} + calculate_hb_expiry_dur()) {
                 my $db = $c->$prefix->db;
 
                 # 0. set 'banned' property of worker to true
@@ -624,7 +632,7 @@ sub register ($self, $app, $config) {
                 }, no_ban => 1);
             }
         }
-        my $duration = Time::HiRes::time - $time;
+        my $duration = get_time() - $time;
         $c->app->log->warn('Channels cleanup took ' . (0 + sprintf("%.2f", $duration)) . ' seconds')
             if $duration > 1;
     });
